package com.link.core.parserplugin.parser;

import com.link.core.parserplugin.AbstractScriptParser;
import com.link.core.parserplugin.util.ParserUtil;
import java.lang.reflect.Method;
import java.util.concurrent.ConcurrentHashMap;
import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.SimpleBindings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JavaScriptParser extends AbstractScriptParser {
   private static Logger logger = LoggerFactory.getLogger(JavaScriptParser.class);
   private final ScriptEngineManager manager = new ScriptEngineManager();
   private final ConcurrentHashMap<String, CompiledScript> expCache = new ConcurrentHashMap();
   private final StringBuffer funcs = new StringBuffer();
   private static int versionCode;
   private final ScriptEngine engine;

   public JavaScriptParser() {
      this.engine = this.manager.getEngineByName("javascript");

      try {
         this.addFunction("hash", ParserUtil.class.getDeclaredMethod("getUniqueHashStr", Object.class));
         this.addFunction("empty", ParserUtil.class.getDeclaredMethod("isEmpty", Object.class));
      } catch (Exception var2) {
         Exception e = var2;
         logger.error(e.getMessage(), e);
      }

   }

   public void addFunction(String name, Method method) {
      try {
         String clsName = method.getDeclaringClass().getName();
         String methodName = method.getName();
         this.funcs.append("function " + name + "(obj){return " + clsName + "." + methodName + "(obj);}");
      } catch (Exception var5) {
         Exception e = var5;
         logger.error(e.getMessage(), e);
      }

   }

   public <T> T getElValue(String exp, Object target, Object[] arguments, Object retVal, boolean hasRetVal, Object userVal, boolean hasUserVal, Class<T> valueType) throws Exception {
      Bindings bindings = new SimpleBindings();
      bindings.put("target", target);
      bindings.put("args", arguments);
      if (hasUserVal) {
         bindings.put("userVal", userVal);
      }

      if (hasRetVal) {
         bindings.put("retVal", retVal);
      }

      CompiledScript script = (CompiledScript)this.expCache.get(exp);
      if (null != script) {
         return script.eval(bindings);
      } else if (this.engine instanceof Compilable) {
         Compilable compEngine = (Compilable)this.engine;
         script = compEngine.compile(this.funcs + exp);
         this.expCache.put(exp, script);
         return script.eval(bindings);
      } else {
         return this.engine.eval(this.funcs + exp, bindings);
      }
   }
}
